package jp.deadend.noname.littlemusic;

import java.io.File;
import java.io.FilenameFilter;
import java.io.FileFilter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.Service;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.os.RemoteException;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

import android.content.ContentResolver;
import android.database.Cursor;
import android.provider.MediaStore;

class SoundFilter implements FilenameFilter {
	public boolean accept(File dir, String name) {
		return Pattern.matches(".*\\.(mp3|ogg|wav)", name);
	}
}

public class LMService extends Service {
	public static final int STOPPED = 0;
	public static final int PLAYING = 1;
	public static final int PAUSED = 2;
	private MediaPlayer mp = new MediaPlayer();
	private ListenLMSInterface mListener;
	private List<String> mSongFiles = new ArrayList<String>();
	private List<String> mSongTitles = new ArrayList<String>();
	private List<String> mDirList = new ArrayList<String>();
	private int mCurrentFileIdx = 0;
	private int mCurrentDirIdx = 0;
	private int mState = STOPPED;
	private boolean mShuffleSong = false;
	private boolean mShuffleDir = false;
	private boolean mLoop = true;
	private String mRootDir;
	private SharedPreferences mSp;

	private BroadcastReceiver mReceiver;
	private boolean isScreenOn = true;
	private boolean hasNotification = false;
	private NotificationManager nm;
	private static final int NOTIFY_ID = 1;
	private static final CharSequence contentTitle = "LittleMusic";
	private PendingIntent contentIntent;
	private Notification mNotification= new Notification();

	private class ScreenReceiver extends BroadcastReceiver {
		@Override public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(Intent.ACTION_SCREEN_OFF)) {
				isScreenOn = false;
			} else if (intent.getAction().equals(Intent.ACTION_SCREEN_ON)) {
				isScreenOn = true;
				if (hasNotification) {
					nm.notify(NOTIFY_ID, mNotification);
					hasNotification = false;
				}
			}
		}
	}

	@Override public void onCreate() {
		super.onCreate();

		nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		contentIntent = PendingIntent.getActivity(this, 0, new Intent(this, LittleMusic.class), 0);
		mNotification.flags = Notification.FLAG_ONGOING_EVENT;

		IntentFilter filter = new IntentFilter(Intent.ACTION_SCREEN_ON);
		filter.addAction(Intent.ACTION_SCREEN_OFF);
		mReceiver = new ScreenReceiver();
		registerReceiver(mReceiver, filter);

		mSp = PreferenceManager.getDefaultSharedPreferences(this);
		loadPrefs();

		updateDirList(mSp.getString(getString(R.string.pref_lastdir), ""));
		updateSongList();
	}

	@Override public void onStart(Intent intent, int startId) {
		isScreenOn = intent.getBooleanExtra("screen_state", true);
	}

	@Override public void onDestroy() {
		mp.stop();
		mp.release();
		nm.cancel(NOTIFY_ID);
		savePrefs();
		unregisterReceiver(mReceiver);
		super.onDestroy();
	}

	@Override public IBinder onBind(Intent intent) {
//		Log.d(getString(R.string.app_name), "service bound");
		return mBinder;
	}

	@Override public void onRebind(Intent intent) {
//		Log.d(getString(R.string.app_name), "service rebound");
		return;
	}

	@Override public boolean onUnbind(Intent intent) {
//		Log.d(getString(R.string.app_name), "service unbound");
		return true;
	}

	private void getDirList(File d) {
		if (d.isDirectory() && d.canRead()) {
			String children[] = d.list();
			String currentSongs[] = d.list(new SoundFilter());
			if (currentSongs.length != 0) {
				mDirList.add(d.getAbsolutePath() + "/");
//				Log.v(getString(R.string.app_name), d.getAbsolutePath() + "/");
			}
			for (int i=0; i<children.length; i++) {
				getDirList(new File(d, children[i]));
			}
		}
	}

	private void updateDirList(String olddir) {
		mDirList.clear();
		getDirList(new File(mRootDir));
		if (mShuffleDir) {
			Collections.shuffle(mDirList);
		} else {
			Collections.sort(mDirList);
		}

		if (mDirList.size() == 0) {
			mDirList.add(mRootDir);
			mCurrentDirIdx = 0;
		} else {
			int idx = mDirList.indexOf(olddir);
			mCurrentDirIdx = (idx == -1) ? 0 : idx;
		}
	}

	private void updateSongList() {
		File path = new File(mDirList.get(mCurrentDirIdx));
		String currentSongs[] = path.list(new SoundFilter());

		mSongFiles.clear();
		if (currentSongs != null) {
			for (String file : currentSongs) {
				mSongFiles.add(file);
//				Log.v(getString(R.string.app_name), file);
			}
			if (mShuffleSong) {
				Collections.shuffle(mSongFiles);
			} else {
				Collections.sort(mSongFiles);
			}
		}
		mCurrentFileIdx = 0;
		updateSongTitlesList();
	}

	private void updateSongTitlesList() {
		mSongTitles.clear();
		if (!mSongFiles.isEmpty()){
			for (String file: mSongFiles) {
			String[] tags = lookupTagsByFileName(file);
			mSongTitles.add(tags[2]);
/*
			try {
				mSongTitles.add(new String(file.getBytes("iso-8859-1"), ENCODING_LIST[fileNameEncoding]));
			} catch (Exception e) {
				Log.e(getString(R.string.app_name), e.getMessage());
			}
*/
			}
		}
	}

	private String[] lookupTagsByFileName(String filename) {
		String[] result = new String[] {"", "", ""};
		String[] columns = new String[] { MediaStore.Audio.Media.ALBUM,
									MediaStore.Audio.Media.ARTIST,
									MediaStore.Audio.Media.TITLE,
									};

		ContentResolver resolver = getContentResolver();
		Cursor cursor = resolver.query(MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
								columns,
								MediaStore.Audio.Media.DISPLAY_NAME + "=?",
								new String[] {filename},
								null);

		if (cursor.moveToFirst()) {
			do {
				for (int i=0; i<3; i++) {
					int idx = cursor.getColumnIndex(columns[i]);
					if (idx == -1) {
						result[i] = "";
					} else {
						result[i] = cursor.getString(idx);
//						try {
//							result[i] = new String(cursor.getBlob(idx), "UTF-8");
//						} catch (Exception e) {}
					}
				}
			} while(cursor.moveToNext());
		} else {
			try {
				result[2] = new String(filename.getBytes("iso-8859-1"), "UTF-8");
			} catch (UnsupportedEncodingException e) {
				Log.e(getString(R.string.app_name), e.getMessage());
			}
		}

		return result;
	}

	private void playSong(String file, String title) {
		try {
			mp.reset();
			mp.setDataSource(file);
			mp.prepare();
			mp.start();

			mp.setOnCompletionListener(new OnCompletionListener() {
				public void onCompletion(MediaPlayer arg0) {
					nextSong();
				}});

			mState = PLAYING;
			setNotification(R.drawable.ic_stat_start, "Playing: " + title);
			if (isScreenOn) {
				nm.notify(NOTIFY_ID, mNotification);
			} else {
				hasNotification = true;
			}
			mListener.posNotify(mCurrentFileIdx);
		} catch (IOException e) {
			stopPlaying();
			Log.e(getString(R.string.app_name), e.getMessage());
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	private void setNotification(int icon, CharSequence contentText) {
		mNotification.icon = icon;
		mNotification.when = System.currentTimeMillis();
		mNotification.setLatestEventInfo(this, contentTitle, contentText, contentIntent);
	}

	private void stopPlaying() {
		mState = STOPPED;
		nm.cancel(NOTIFY_ID);
		mp.stop();
		try {
			mListener.posNotify(-1);
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	private void nextSong() {
		if (!mSongFiles.isEmpty()) {
			mCurrentFileIdx++;
			if (mCurrentFileIdx >= mSongFiles.size()) {
				if (mLoop) {
					mCurrentFileIdx = 0;
				} else {
					nextDir();
					return;
				}
			}
			playSong(mDirList.get(mCurrentDirIdx) + mSongFiles.get(mCurrentFileIdx),
			mSongTitles.get(mCurrentFileIdx));
		}
	}

	private void prevSong() {
		if (!mSongFiles.isEmpty()) {
			if (mp.getCurrentPosition() < 5000 && mCurrentFileIdx >= 1) {
				mCurrentFileIdx--;
			}
			playSong(mDirList.get(mCurrentDirIdx) + mSongFiles.get(mCurrentFileIdx),
			mSongTitles.get(mCurrentFileIdx));
		}
	}

	private void nextDir() {
		nm.cancel(NOTIFY_ID);
		mp.stop();
		mCurrentDirIdx++;
		if (mCurrentDirIdx >= mDirList.size()) {
			mCurrentDirIdx = 0;
		}
		updateSongList();
		try {
 			mListener.dirNotify(mDirList.get(mCurrentDirIdx));
			mListener.songListNotify(mSongTitles);
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}

		switch (mState) {
		case PLAYING:
			if (!mSongFiles.isEmpty()) {
				playSong(mDirList.get(mCurrentDirIdx) + mSongFiles.get(mCurrentFileIdx), mSongTitles.get(mCurrentFileIdx));
			}
			break;
		case PAUSED:
			stopPlaying();
			break;
		case STOPPED:
			break;
		default:
			break;
		}
	}

	private void loadPrefs() {
		mShuffleSong = mSp.getBoolean(getString(R.string.pref_shufflesong), false);
		mShuffleDir = mSp.getBoolean(getString(R.string.pref_shuffledir), false);
		mLoop = mSp.getBoolean(getString(R.string.pref_loop), true);
		mRootDir = mSp.getString(getString(R.string.pref_rootdir), "/sdcard");
	}

	private void savePrefs() {
		SharedPreferences.Editor e = mSp.edit();

		e.putBoolean(getString(R.string.pref_shufflesong), mShuffleSong);
		e.putBoolean(getString(R.string.pref_shuffledir), mShuffleDir);
		e.putBoolean(getString(R.string.pref_loop), mLoop);
		e.putString(getString(R.string.pref_lastdir), mDirList.get(mCurrentDirIdx));
		e.putString(getString(R.string.pref_rootdir), mRootDir);
		e.commit();
	}

	private final LMSInterface.Stub mBinder = new LMSInterface.Stub() {
		public List<String> getSongs() {
//			return mSongFiles;
			return mSongTitles;
		}
		public List<String> getDirs() {
			return mDirList;
		}

		public int getPos() {
			return mCurrentFileIdx;
		}
		public String getCurrentDir() {
			return mDirList.get(mCurrentDirIdx);
		}

		public void playFile(int position) throws RemoteException {
			if (!mSongFiles.isEmpty()) {
				try {
					mCurrentFileIdx = position;
					playSong(mDirList.get(mCurrentDirIdx) + mSongFiles.get(mCurrentFileIdx), mSongTitles.get(mCurrentFileIdx));
				} catch (IndexOutOfBoundsException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}
		}

		public int pause() throws RemoteException {
			if (mState == PLAYING) {
				mState = PAUSED;
				setNotification(R.drawable.ic_stat_pause, "Paused");
				nm.notify(NOTIFY_ID, mNotification);
				mp.pause();
			} else if (mState == PAUSED) {
				mState = PLAYING;
				setNotification(R.drawable.ic_stat_start, "Playing: " + mSongTitles.get(mCurrentFileIdx));
				nm.notify(NOTIFY_ID, mNotification);
				mp.start();
			}
			return mState;
		}

		public void stop() throws RemoteException {
			if (mState == PLAYING || mState == PAUSED) {
				stopPlaying();
			}
		}
		public void skipBack() throws RemoteException {
			if (mState == PLAYING || mState == PAUSED) prevSong();
		}
		public void skipForward() throws RemoteException {
			if (mState == PLAYING || mState == PAUSED) nextSong();
		}
		public void skipDir() throws RemoteException {
			nextDir();
		}
		public void gotoDir(int idx) throws RemoteException {
			if (idx <= mDirList.size() && idx >= 0) {
				mCurrentDirIdx = idx;
			} else {
				return;
			}
			stopPlaying();
			updateSongList();
			try {
				mListener.dirNotify(mDirList.get(mCurrentDirIdx));
				mListener.songListNotify(mSongTitles);
			} catch (RemoteException e) {
				Log.e(getString(R.string.app_name), e.getMessage());
			}
		}

		public void setShuffleSong(boolean val) throws RemoteException {
			if (mShuffleSong != val){
				mShuffleSong = val;
				String old_current = mSongFiles.get(mCurrentFileIdx);
				if (mShuffleSong) {
					Collections.shuffle(mSongFiles);
				} else {
					Collections.sort(mSongFiles);
				}
				updateSongTitlesList();
				mCurrentFileIdx = mSongFiles.indexOf(old_current);
				try {
					mListener.songListNotify(mSongTitles);
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}
		}
		public void setShuffleDir(boolean val) throws RemoteException {
			if (mShuffleDir != val) {
				mShuffleDir = val;
				String old_current = mDirList.get(mCurrentDirIdx);
				if (mShuffleDir) {
					Collections.shuffle(mDirList);
				} else {
					Collections.sort(mDirList);
				}
				mCurrentDirIdx = mDirList.indexOf(old_current);
			}
		}
		public void setLoop(boolean val) throws RemoteException {
			mLoop = val;
		}
		public void setRootDir(String d) throws RemoteException {
			String old_current = mDirList.get(mCurrentDirIdx);
			mRootDir = d;
			mSp.edit().putString(getString(R.string.pref_rootdir), mRootDir).commit();

			updateDirList(old_current);
			if (!old_current.equals(mDirList.get(mCurrentDirIdx))) {
				stopPlaying();
				updateSongList();
				try {
					mListener.dirNotify(mDirList.get(mCurrentDirIdx));
					mListener.songListNotify(mSongTitles);
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}
		}

		public boolean getShuffleSong() throws RemoteException {
			return mShuffleSong;
		}
		public boolean getShuffleDir() throws RemoteException {
			return mShuffleDir;
		}
		public boolean getLoop() throws RemoteException {
			return mLoop;
		}
		public int getState() throws RemoteException {
			return mState;
		}
		public String getRootDir() throws RemoteException {
			return mRootDir;
		}

		public void setListener(ListenLMSInterface listenInterface) throws RemoteException {
			mListener = listenInterface;
		}
	};
}