package jp.deadend.noname.littlemusic;

import java.util.Collections;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.ToggleButton;

public class LittleMusic extends ListActivity implements ServiceConnection {
	private static final int MESSAGE_SONGLIST = 1;
	private static final int MESSAGE_DIR = 2;
	private static final int MESSAGE_POS = 3;
	private static final int DIALOG_PROGRESS = 0;
	private static final int DIALOG_ROOTDIR = 1;

	private List<String> dispSongs;
	private LMSInterface lmInterface = null;
	private boolean isConnected = false;
//	private String[] enclist;

	private final ListenLMSInterface.Stub listenInterface = new ListenLMSInterface.Stub() {
		@Override public void songListNotify(List<String> songs) throws RemoteException {
			listenHandler.sendMessage(listenHandler.obtainMessage(MESSAGE_SONGLIST, songs));
		}
		@Override public void dirNotify(String currentDir) throws RemoteException {
			listenHandler.sendMessage(listenHandler.obtainMessage(MESSAGE_DIR, currentDir));
		}
		@Override public void posNotify(int pos) throws RemoteException {
			listenHandler.sendMessage(listenHandler.obtainMessage(MESSAGE_POS, pos));
		}
	};

	private final Handler listenHandler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			switch(msg.what) {
				case MESSAGE_SONGLIST:
					SongListAdapter songList = new SongListAdapter(getApplicationContext(), R.layout.song_item, (List<String>)msg.obj);
					setListAdapter(songList);
					break;
				case MESSAGE_DIR:
					TextView tview = (TextView) findViewById(R.id.text_pwd);
					tview.setText((String)msg.obj);
					break;
				case MESSAGE_POS:
					SongListAdapter adapter = (SongListAdapter)getListAdapter();
					adapter.setHighlightPos((Integer)msg.obj);
					adapter.notifyDataSetChanged();
					break;
				default:
					super.handleMessage(msg);
					break;
			}
		}
	};

	@Override
	protected Dialog onCreateDialog(int id) {
		final Dialog dialog;
		switch (id) {
		case DIALOG_PROGRESS:
			dialog = new ProgressDialog(this);
			((ProgressDialog)dialog).setMessage(getResources().getString(R.string.label_loading));
			((ProgressDialog)dialog).setCancelable(false);
			((ProgressDialog)dialog).setIndeterminate(true);
			break;
		case DIALOG_ROOTDIR:
			final EditText editText = new EditText(this);
			dialog = new AlertDialog.Builder(this)
				.setTitle(R.string.label_dialog_rootdir_title)
				.setView(editText)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						String str = editText.getText().toString();
					}
				})
				.create();
			break;
		default:
			dialog = null;
		}

		return dialog;
	}

	@Override public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.main);

		ImageButton ibutton = (ImageButton) findViewById(R.id.button_stop);
		ibutton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					lmInterface.stop();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		ibutton = (ImageButton) findViewById(R.id.button_pause);
		ibutton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					lmInterface.pause();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		ibutton = (ImageButton) findViewById(R.id.button_skipb);
		ibutton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					lmInterface.skipBack();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		ibutton = (ImageButton) findViewById(R.id.button_skipf);
		ibutton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					lmInterface.skipForward();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		ibutton = (ImageButton) findViewById(R.id.button_skipd);
		ibutton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					lmInterface.skipDir();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		Button button = (Button) findViewById(R.id.button_go);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					final List<String> dirs = lmInterface.getDirs();
					final List<String> dirs_sorted = new ArrayList<String> (dirs);
					Collections.sort(dirs_sorted);
					new AlertDialog.Builder(LittleMusic.this)
						.setTitle("Change Dir")
						.setItems(dirs_sorted.toArray(new String[0]), new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int item) {
								try {
									lmInterface.gotoDir(dirs.indexOf(dirs_sorted.get(item)));
								} catch (RemoteException e) {
									Log.e(getString(R.string.app_name), e.getMessage());
								}
							}})
						.show();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		button = (Button) findViewById(R.id.button_root);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					final EditText editText = new EditText(LittleMusic.this);
					editText.setText(lmInterface.getRootDir());

					new AlertDialog.Builder(LittleMusic.this)
						.setTitle("Set Root Dir")
						.setView(editText)
						.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {
								try {
									lmInterface.setRootDir(editText.getText().toString());
								} catch (RemoteException e) {
									Log.e(getString(R.string.app_name), e.getMessage());
								}
							}})
						.show();
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		final ToggleButton button_ssong = (ToggleButton) findViewById(R.id.button_ssong);
		button_ssong.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try{
					if (button_ssong.isChecked()) {
						lmInterface.setShuffleSong(true);
					} else {
						lmInterface.setShuffleSong(false);
					}
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		final ToggleButton button_sdir = (ToggleButton) findViewById(R.id.button_sdir);
		button_sdir.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					if (button_sdir.isChecked()) {
						lmInterface.setShuffleDir(true);
					} else {
						lmInterface.setShuffleDir(false);
					}
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});
		final ToggleButton button_loop = (ToggleButton) findViewById(R.id.button_loop);
		button_loop.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				try {
					if (button_loop.isChecked()) {
						lmInterface.setLoop(true);
					} else {
						lmInterface.setLoop(false);
					}
				} catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			}});

		showDialog(DIALOG_PROGRESS);
		startService(new Intent(this, LMService.class));
	}

	@Override protected void onResume() {
		if(isConnected == false) {
//			bindService(new Intent(this, LMService.class),  this, Context.BIND_AUTO_CREATE);
//			startService(new Intent(this, LMService.class));
			if (!bindService(new Intent(this, LMService.class),  this, 0)) {
				showDialog(DIALOG_PROGRESS);
				startService(new Intent(this, LMService.class));
				bindService(new Intent(this, LMService.class),  this, 0);
			}
			isConnected = true;
		}
		super.onResume();
	}

	@Override protected void onPause() {
		int state = LMService.STOPPED;
		try {
			state = lmInterface.getState();
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
		unbindService(LittleMusic.this);
		if (state == LMService.STOPPED) stopService(new Intent(this, LMService.class));
		isConnected = false;

		super.onPause();
	}

	@Override public void onServiceConnected(ComponentName className, IBinder service) {
		lmInterface = LMSInterface.Stub.asInterface(service);
		updateDisplaySongList();
		removeDialog(DIALOG_PROGRESS);
		int state = LMService.STOPPED;
		try {
			lmInterface.setListener(listenInterface);

			ToggleButton tbutton = (ToggleButton) findViewById(R.id.button_ssong);
			tbutton.setChecked(lmInterface.getShuffleSong());
			tbutton = (ToggleButton) findViewById(R.id.button_sdir);
			tbutton.setChecked(lmInterface.getShuffleDir());
			tbutton = (ToggleButton) findViewById(R.id.button_loop);
			tbutton.setChecked(lmInterface.getLoop());

			TextView tview = (TextView) findViewById(R.id.text_pwd);
			tview.setText(lmInterface.getCurrentDir());

			state = lmInterface.getState();
			if (state != LMService.STOPPED) {
				SongListAdapter adapter = (SongListAdapter)getListAdapter();
				adapter.setHighlightPos(lmInterface.getPos());
				adapter.notifyDataSetChanged();
			}
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	@Override public void onServiceDisconnected(ComponentName className) {
		lmInterface = null;
	}

	@Override protected void onListItemClick(ListView l, View v, int position, long id) {
		try {
			if (!isConnected) {
//				startService(new Intent(this, LMService.class));
//				bindService(new Intent(this, LMService.class),  this, 0);
				if (!bindService(new Intent(this, LMService.class),  this, 0)) {
					showDialog(DIALOG_PROGRESS);
					startService(new Intent(this, LMService.class));
					bindService(new Intent(this, LMService.class),  this, 0);
				}
				isConnected = true;
			}
			lmInterface.playFile(position);
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	public void updateDisplaySongList() {
		try {
			dispSongs = lmInterface.getSongs();
			if (dispSongs != null) {
				SongListAdapter songList = new SongListAdapter(this, R.layout.song_item, dispSongs);
				setListAdapter(songList);
			}
		} catch (RemoteException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	static class SongListAdapter extends ArrayAdapter<String> {
		private static final int HIGHLIGHT_COLOR = 0xFF00FFFF;
		private LayoutInflater mInflater;
		private int mHighlightPos = -1;

		public SongListAdapter(Context context, int textViewResourceId, List<String> objects) {
			super(context, textViewResourceId, objects);
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View view;
			TextView tv;

			if (convertView == null) {
				view = mInflater.inflate(R.layout.song_item, null);

				tv = (TextView)view.findViewById(R.id.text1);
				view.setTag(tv);
			} else {
				view = convertView;
				tv = (TextView)view.getTag();
			}

			tv.setText(getItem(position));
			if (position == mHighlightPos) {
				tv.setTextColor(0xFF000000);
				tv.setBackgroundColor(HIGHLIGHT_COLOR);
			} else {
				tv.setTextColor(0xFFFFFFFF);
				tv.setBackgroundColor(0xFF000000);
			}

			return view;
		}

		public void setHighlightPos(int pos) {
			mHighlightPos = pos;
		}
	}
}
